# -*- coding: utf-8 -*-
from plone.app.relationfield.testing import FUNCTIONAL_WIDGETS_TESTING
from plone.testing import layered

import doctest
import os
import unittest


optionflags = (doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests([
        layered(doctest.DocFileSuite(
                os.path.join(os.path.pardir, 'schemaeditor.txt'),
                optionflags=optionflags),
                FUNCTIONAL_WIDGETS_TESTING)
    ])
    return suite


if __name__ == '__main__':
    unittest.main(default='test_suite')



# -*- coding: utf-8 -*-
from plone.app.relationfield.testing import Address
from plone.app.relationfield.testing import FUNCTIONAL_TESTING
from plone.app.relationfield.testing import IPerson
from plone.app.relationfield.testing import Person
from plone.app.relationfield.widget import RelationListDictDataManager
from z3c.form.interfaces import IDataManager
from z3c.relationfield.interfaces import IRelationValue
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import unittest2 as unittest


class RelationListDictDataManagerTest(unittest.TestCase):
    layer = FUNCTIONAL_TESTING

    def setUp(self):
        portal = self.layer['portal']
        intids = getUtility(IIntIds)

        self.person = portal.person = person = Person('Roel Bruggink')
        person.__parent__ = portal

        self.addresses = person.addresses = addresses = []
        for streetname in ['Jansbinnensingel', 'Willemsplein']:
            address = Address(streetname, 'Arnhem')
            addresses.append(address)

            # five.intids' register expect aq wrapped objects, but __parent__
            # works just fine.
            address.__parent__ = portal

            intids.register(address)

    def test_get_datamanger(self):
        dm = getMultiAdapter(({}, IPerson['addresses'], ), IDataManager)
        self.assertTrue(isinstance(dm, RelationListDictDataManager))

    def test_datamanager_get_empty(self):
        dm = RelationListDictDataManager({}, IPerson['addresses'])
        self.assertEqual(dm.get(), [])

    def test_datamanager_set_empty(self):
        dm = RelationListDictDataManager({}, IPerson['addresses'])
        dm.set([])
        self.assertEqual(dm.get(), [])

    def test_datamanager_set_nonempty(self):
        dm = RelationListDictDataManager({}, IPerson['addresses'])
        dm.set(self.person.addresses)
        self.assertEqual(dm.get(), self.person.addresses)

    def test_datamanager_should_contain_relationvalues(self):
        dm = RelationListDictDataManager({}, IPerson['addresses'])
        dm.set(self.person.addresses)

        storage = dm.data['addresses']
        self.assertNotEqual(storage, [])
        self.assertNotEqual(filter(IRelationValue.providedBy, storage), [])



# -*- coding: utf-8 -*-
from plone.testing import layered
from plone.testing.zca import ZCMLSandbox
from unittest import TestSuite

import doctest
import plone.app.relationfield.tests


ZCML_SANDBOX = ZCMLSandbox(
    filename='test_marshall.zcml',
    package=plone.app.relationfield.tests
)


def test_suite():
    suite = TestSuite()
    OPTIONFLAGS = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)
    suite.addTest(layered(
        doctest.DocFileSuite(
            '../marshaler.rst',
            optionflags=OPTIONFLAGS,
            package='plone.app.relationfield.tests',
        ),
        layer=ZCML_SANDBOX)
    )
    return suite



# -*- coding: utf-8 -*-
from plone.rfc822.defaultfields import BaseFieldMarshaler
from z3c.relationfield import RelationValue


class RelationFieldMarshaler(BaseFieldMarshaler):
    """Field marshaler for z3c.relationfield IRelation and IRelationChoice
    fields
    """

    ascii = True

    def encode(self, value, charset='utf-8', primary=False):
        if value is None:
            return None
        return str(value.to_id)

    def decode(
        self,
        value,
        message=None,
        charset='utf-8',
        contentType=None,
        primary=False,
    ):
        try:
            toId = int(value.decode(charset))
        except TypeError, e:
            raise ValueError(e)
        return RelationValue(toId)



# -*- coding: utf-8 -*-
from five.intid.intid import IntIds
from five.intid.site import addUtility
from z3c.relationfield.index import RelationCatalog
from zc.relation.interfaces import ICatalog
from zope.intid.interfaces import IIntIds


def add_relations(context):
    addUtility(context, ICatalog, RelationCatalog, ofs_name='relations',
               findroot=False)


def add_intids(context):
    addUtility(context, IIntIds, IntIds, ofs_name='intids',
               findroot=False)


def installRelations(context):
    if context.readDataFile('install_relations.txt') is None:
        return
    portal = context.getSite()
    add_relations(portal)
    return 'Added relations utility.'



# -*- coding: utf-8 -*-
from plone.app.relationfield.monkey import PATCHES

import pkg_resources


PATCHES


try:
    pkg_resources.get_distribution('plone.app.contenttypes')
except pkg_resources.DistributionNotFound:
    HAS_CONTENTTYPES = False
else:
    HAS_CONTENTTYPES = True



# -*- coding: utf-8 -*-
from plone.app.vocabularies.catalog import CatalogSource
from plone.supermodel.exportimport import BaseHandler
from plone.supermodel.utils import valueToElement
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema


class RelationChoiceBaseHandler(BaseHandler):

    filteredAttributes = BaseHandler.filteredAttributes.copy()
    filteredAttributes.update({
        'portal_type': 'w',
        'source': 'rw',
        'vocabulary': 'rw',
        'vocabularyName': 'rw'
    })

    def __init__(self, klass):
        super(RelationChoiceBaseHandler, self).__init__(klass)

        self.fieldAttributes['portal_type'] = \
            schema.List(__name__='portal_type',
                        title=u'Allowed target types',
                        value_type=schema.Text(title=u'Type'))

    def _constructField(self, attributes):
        portal_type = attributes.get('portal_type') \
            or attributes.get('portal_types') \
            or []
        if 'portal_type' in attributes:
            del attributes['portal_type']

        if not portal_type:
            attributes['source'] = CatalogSource()
        else:
            attributes['source'] = CatalogSource(portal_type=portal_type)

        return super(RelationChoiceBaseHandler,
                     self)._constructField(attributes)

    def write(self, field, name, type, elementName='field'):
        element = super(RelationChoiceBaseHandler,
                        self).write(field, name, type, elementName)
        portal_type = []

        portal_type.extend(
            field.source.query.get('portal_type') or [])

        if portal_type:
            attributeField = self.fieldAttributes['portal_type']
            child = valueToElement(attributeField, portal_type,
                                   name='portal_type', force=True)
            element.append(child)

        return element

RelationChoiceHandler = RelationChoiceBaseHandler(RelationChoice)
RelationListHandler = BaseHandler(RelationList)
